#!/usr/bin/perl
# ===============================================================
# Partition Space Monitor and Culler
# "The Culler of Authority"
# Version 1.0 SjG 28 Dec 2002
# samuelg@fogbound.net
# ===============================================================
#      **WARNING**  **WARNING**  **WARNING**  **WARNING**
#
# THIS PROGRAM DELETES FILES! THIS PROGRAM COULD DELETE IMPORTANT
# FILES IF NOT CONFIGURED CORRECTLY!
#
#      **WARNING**  **WARNING**  **WARNING**  **WARNING**
#
# This is built based upon the output of a few utility
# programs, which should be fairly standard, such as "df" and
# "find". It's all built based on the output from these programs
# as they are distributed as part of the Debian Woody release.
#
# While the format of the output from these programs should be
# relatively similar across distros (especially if you use the GNU
# versions), they may vary, so use this program with caution!
#
#   IF THE FORMATS ARE DIFFERENT, THIS COULD CAUSE THIS
#   PROGRAM TO DELETE IMPORTANT FILES!
#
# ALWAYS run with the "-t" flag the first time to make sure your
# configuration is correct, and to avoid deleting important data!
# ===============================================================
#
# ===============================================================
# Configuration:
#
# Filesystems to monitor. If you'd like to monitor more than one,
# separate them with commas.
# @filesystems = ("/dev/hda0","/dev/hda1");

@filesystems = ("/dev/hda0");

# Directories to cull. If space gets tight, which directories
# should we cull old files from? Note that these are the tops of
# the hierarchies -- any files in directories below this point
# are eligible for deletion!
# You can list multiple directories, but if they're on the same
# partition, it's possible that only the first will get culled.
# @culldirs = ("/home/motion/capture/front_door","/home/motion/capture/back_yard");

@culldirs = ("/home/motion/capture_root/capture/");

# preservedirs are directories which will not be deleted, even if they
# are empty. This way, Motion won't lose a place to put the data.
@preservedirs = ("/home/motion/capture_root/capture/cam1","/home/motion/capture_root/capture/cam2","/home/motion/capture_root/capture/cam3","/home/motion/capture_root/capture/cam4","/home/motion/capture_root/capture/cam5","/home/motion/capture_root/capture/cam6");

# Trigger threshold. How full should a partition be allowed to get
# (interpreted as a percentage) before culling begins? Must be
# greater than 0!
# $threshold = 80;

$threshold = 80;

#
# Be noisy?
# Set this nonzero to output telemetry to STDOUT. If you run in test mode,
# this will be over-ruled, and set to true.
$verbose = 1;

# ---------------------------------------------------------------
# Nothing to see beyond this point, unless you need to change
# the code to work with other output formats of Unix commands.
#
# Oh -- and while you're thinking about it, put me in a crontab!
# ---------------------------------------------------------------

# What command should I use for "df" ?
$DFCOMMAND = "df -P -k";

# should I use "xargs" and bulk deletions for improved efficiency?
# set to 0 to not use xargs, or 1 to use xargs.
$USEXARGS = 1;
# what should I assume maximum command line length is? 4096 is a safe
# bet (what with POSIX and all), although BASH supposedly can buffer
# any arbitrary length. Anyway, call it 4000 for safety.
$MAXCOMMANDLINE = 4000;

# ---------------------------------------------------------------
# This is really the end of configurable stuff. Really. Well,
# unless you want to tweak the code, that is.
# ---------------------------------------------------------------
# snag any arguments
*theArgs = &grabArgs(@ARGV);

# test implies verbose
if ($theArgs{'t'})
{
   $verbose = 1;
}

# load our filesystem and culldir arrays into hashes for easier manipulation
foreach (@filesystems)
        {
          $fs{$_} = 1;
        }
foreach (@culldirs)
        {
          $cd{$_} = 1;
        }

# test to see if it's time to cull
@df = split(/\n/,`$DFCOMMAND`);
foreach $thisLine (@df)
{
  if ($thisLine =~ /^Filesystem/i)
  {
    next;
  }
  @thisFilesystem = split(/\s+/,$thisLine);

  $thisFs = $thisFilesystem[0];

  # populate a bunch of stuff about this filesystem
  $blocks{$thisFs} = $thisFilesystem[1];
  $used{$thisFs} = $thisFilesystem[2];
  ($utilized{$thisFs} = $thisFilesystem[4]) =~ s/%//;
  $mountPoint{$thisFs} = $thisFilesystem[5];
  $device{$thisFilesystem[5]} = $thisFs;
  $deviceNo{stat($thisFilesystem[5])} = $thisFs;
  if ($utilized{$thisFs} > 0)
  {
  	$blocksPerPercent{$thisFs} = $used{$thisFs} / $utilized{$thisFs};
  }
  else
  {
  	# dummy. Should never matter. Famous last words.
  	$blocksPerPercent{$thisFs} = 1;
  }

  # simple check to make sure we're sane.
  # Doh. Can't do this because 'df' is not sane!
  #if (int(1000*$used{$thisFs}/$blocks{$thisFs})/10 != $utilized{$thisFs})
  #{
  # die "can't understand 'df' format!\n";
  #}

  if ($fs{$thisFs} == 1)
  {
    # at quota level?
    if ($verbose)
    {
      print "Filesystem $thisFs is at $utilized{$thisFs}\%\n";
    }
    if ($utilized{$thisFs} > $threshold)
    {
      # mark this filesystem for culling
      $fs{$thisFs} = 2;
      #since df uses different means of computing "total blocks" versus
      # "available" and "used," we'll use the "used" number for consistency's
      # sake.
      $toRemove{$thisFs} =  ($utilized{$thisFs} - $threshold) * $blocksPerPercent{$thisFs};;
      if (1 || $verbose)
      {
        print "$thisFs to be culled. $toRemove{$thisFs} blocks to be freed.\n";
      }
    }
    else
    {
      if (1 || $verbose)
      {
        print "$thisFs will not be culled until use exceeds $threshold\% utilization.\n";
      }
    }
  }
}

# the loop that follows could be rolled into the "if utilized"
# clause above, and maybe it should. I had my mysterious reasons
# for doing it this way (i.e., laziness), but perhaps this should
# be revisited.

# now, figure out which directories need to have files removed.
foreach $thisDir (keys(%cd))
{
   if ($fs{$deviceNo{stat($thisDir)}} == 2)
   {
     # this directory is on a filesystem that's marked for culling.
     $cd{$thisDir} = 2;
     if ($verbose)
     {
       print "Culling oldest files from $thisDir.\n";
     }
     # big gnarly find command. GNU-find is practically an OS in and of itself.
     @fileList = split(/\n/, `find $thisDir -type f -printf '%T@ %k %p\n' | sort -n`);
     $rmKilobytes = 0;
     $index = 0;
     $removeList = "";
     while ($rmKilobytes < $toRemove{$thisFs} && $index < $#fileList)
     {
       ($timestamp, $size, $name) = split(/\s/,$fileList[$index]);
       $rmKilobytes += $size;
	 if ($USEXARGS)
	 {
 	if ($verbose)
	{
		print "Adding $name to removal list\n";
	}
        # heavy handed, but safe.
         ($safename = $name) =~ s/([\W])/\\$1/g;
         $removeList .= "$safename ";
         if (length($removeList) > $MAXCOMMANDLINE)
         {
           if ($verbose)
           {
              print "Issuing: rm $removeList\n";
           }
           if ($theArgs{'t'})
           {
             print "(Testmode. No command issued.)\n";
           }
             else
             {
            $result = `rm $removeList`;
             }
            $removeList = "";
         }
  }
  else
  {
	if ($verbose)
	{
		print "removing $name\n";
	}

           if ($theArgs{'t'})
           {
             print "(Testmode. No command issued.)\n";
           }
             else
             {
		unlink $name;
           }
  }
       $index++;
     }



   # remove any empty directories left by the culling.
   # Using "find" again, and using rmdir since it'll only
   # remove empty directories. crude, but effective.
   if ($USEXARGS)
   {
   $res = "find $thisDir -type d -ctime +1 ";
if ($#preservedirs > 0)
{
$res .= '-not \( ';
   foreach (@preservedirs)
	{
	$res .= "-path \"$_\" -or ";
	}
$res .= '-false \) ';
}
   $res .= "-print0 | sort -rz | xargs -0 rmdir";
   `$res`;
print "Issued: $res\n";
   }
   else
   {
        $res = `find $thisDir -type d -exec rmdir {}\;`;
   }
   }
}


exit(0);


sub grabArgs {
	my (@args) = @_;
	%tmp = ();
	my ($last) = '';
	foreach $this (@args)
		{
		if (substr($this,0,1) eq '-')
			{
			if (length($last) > 0)
				{
				$tmp{$last} = $last;
				$last = substr($this,1);
				}
			else
				{
				$last = substr($this,1);
				}
			}
		else
			{
			if (length($last) > 0)
				{
				$tmp{$last} = $this;
				$last = '';
				}
			else
				{
				$tmp{'none'} .= " ".$this;
				}
			}
		}
		if (length($last) > 0)
			{
			$tmp{$last} = $last;
			}

	return *tmp;
}



1;

